#
#   GenerateATX.py
#
# inputs:   <name>
#           '.\<name>.h'
#
# output:   '..\<name>.atx'
#
# Requires Python 2.0. (http://www.activestate.com/Products/ActivePython/index.html)
#

import os
import sys
import re
import string
import glob
from stat import *
from _ShiftersPaths import *

kLogFile        = 'GenerateATX.txt'
kAsbHeaderSize  = 48
kAsbListTab     = kShiftersRuntimePath + '.\\script\\asblist.tab'
#kMaxFormSize    = 173056

gFilesNotFound  = []
gSubFormAsbs       = (
	"AutoG",
	"AutoM",
	"AutoS",
	"AutoW",
	"BeastG",
	"BeastM",
	"BeastS",
	"BeastW",
	"GenieG",
	"GenieM",
	"GenieS",
	"GenieW",
	"KreeganG",
	"KreeganM",
	"KreeganS",
	"KreeganW",
	"HybridG",
	"HybridM",
	"HybridS",
	"HybridW",
	"UndeadG",
	"UndeadM",
	"UndeadS",
	"UndeadW"
    )

gFormAsbs       = (
	"Auto",
	"Beast",
	"Genie",
	"Kreegan",
	"Hybrid",
	"Undead",
    )


def Log_Reset():
    f = open( kLogFile, 'w' )
    f.close()

def Log(str):
    f = open( kLogFile, 'a' )
    stdout_saved = sys.stdout
    sys.stdout = f

    print str

    sys.stdout = stdout_saved
    f.close()

def GenerateATX(name):
    
    base = os.path.splitext(os.path.basename(name))[0]
    filename    = kSoundPS2TempPath + base + '.h'
    atxfilename = kSoundPS2Path + base + '.atx'

    if ( os.path.exists(atxfilename) ) :
        bCanWrite = os.access(atxfilename,os.W_OK)
        if not bCanWrite:
            Log( 'WARNING:\t' + atxfilename + ' is Read-Only.' )
            return
    
    if ( os.path.exists(filename) ) :
        Log(  'Processing:\t' + filename + ',' + ' -> ' + atxfilename )
        f_in = open( filename, 'r' )
        f_out = open(atxfilename,'w')
        for line in f_in.readlines():
            line = string.strip(line)
            if ( re.match("^#define",line) ):
              words = line.split()
              constantname = words[1].lower()
              if (  (not(re.match(".*basewaveindex",constantname))) \
                and (not(re.match(".*patch_size",constantname))) \
                and (not(re.match(".*numsamples",constantname))) \
                and (not(re.match(".*basepatchindex",constantname))) ):
                  index = words[2]
                  n = constantname.find('_') + len(base) + 1
                  tmp = constantname[(n+1):]
                  f_out.write( tmp + '\t' + index + '\n' )

        f_in.close()
        f_out.close()

def GetAsbFileSize( filename ):
    filename = kSoundPS2Path + filename + '.asb'
    size = 0
    if os.path.exists(filename) :
        size = os.stat(filename)[ST_SIZE] - kAsbHeaderSize
    else:
        if filename not in gFilesNotFound:
            gFilesNotFound.append(filename)
    return size

def GetAsbFileListForCap( capname ):
    result = []

    if not os.path.exists(kAsbListTab) :
        Log( kAsbListTab + ' NOT FOUND' )
        return result

    f = open( kAsbListTab, 'r' )
    for line in f.readlines():
        fields = string.split(string.lower(string.strip(line)),'\t')
        if ( len( fields) > 1 ):
            if ( cmp( capname, fields[0] ) == 0 ):
                asbs = fields[1:]
                for asb in asbs:
                    result.append(asb)
                break
    f.close()
    
    return result

def GetAsbFileSizeForCap( capname ):
    if not os.path.exists(kAsbListTab) :
        Log( kAsbListTab + ' NOT FOUND' )
        return GetAsbFileSize( capname )
    
    size = 0
    asbs = GetAsbFileListForCap( capname )
    for asb in asbs:
        size = size + GetAsbFileSize( asb )

    return size

def GetFormAsbName( subformName ):
	form = ""
	for form_ in gFormAsbs:
		matchstr = form_ + '*'
		if ( re.match( matchstr, subformName) ):
			form = form_
	return form

def GetMaxFormPlusSubFormAsbSize():
	max = 0
	for asb in gSubFormAsbs:
		size = GetAsbFileSize( asb ) + GetAsbFileSize( GetFormAsbName( asb ) )
		if ( size > max ):
			max = size
	return max

def GetMaxSubFormAsbName():
    max = 0
    name = ""
    for asb in gSubFormAsbs:
        size = GetAsbFileSize( asb )
        if ( size > max ):
            max = size
            name = asb
    return name

def GetMaxMenuAsbSize():
	ingamesize = GetAsbFileSize( "ingame" )
	#max = GetAsbFileSize( "menu" )
	#if ( ingamesize > max ):
	#	max = ingamesize
	max = ingamesize
	return max


def Verify_ATXs():
    
    #
	# Build list of levels
	# 
    f_levels = open(kCapList)
    levels = []        
    for line in f_levels.readlines():
        level = string.split(string.lower(string.strip(line)),'\t')[0]
        if not level in levels:
            levels.append(level)
    f_levels.close()
    levels.sort()


    #
	# Report ASB Sizes
	#
	   
    #TotalSPURAM = 2097152 - 186904
    TotalSPURAM = 2097152 - 0

    ui_size     = GetAsbFileSize( 'ui' )
    global_size = GetAsbFileSize( 'global' )
    #misc_size   = GetAsbFileSize( 'misc' )

    max_form_size   = GetMaxFormPlusSubFormAsbSize()
    max_menu_size   = GetMaxMenuAsbSize()

    maxsubform = GetMaxSubFormAsbName()
     
    Log( "" )
    Log( "================================================" )
    Log( 'Total SPU RAM:\t\t' + str(TotalSPURAM) + ' bytes' )
    Log( "================\t==============" )
    Log( "ui.asb\t\t\t"     + string.rjust(str(ui_size),7) )
    Log( "global.asb\t\t"   + string.rjust(str(global_size),7) )
    Log( "Max Form+SubForm asb\t" + string.rjust(str(max_form_size),7) + '\t(' + GetFormAsbName(maxsubform)+ ' + ' + maxsubform +  ')' )
    Log( "Max Menu size \t\t" +  string.rjust(str(max_menu_size),7) + '\t(ingame.asb)' )

    AvailableSPURAM = TotalSPURAM - (ui_size+global_size+max_form_size+max_menu_size)
    Log( "================\t==============" )
    Log( "Level+Entities Budget:\t" +  string.rjust(str(AvailableSPURAM),7) + " bytes" )
    Log( "======================================" )
   
    for level in levels:
        size = GetAsbFileSizeForCap( level )
        if ( size > 0 ):
            asblist = GetAsbFileListForCap( level )
            #asbstr = string.join( asblist, ", ")
            #asbstr = '(' + asbstr + ')'
            strbuf = string.ljust(level,8)
            strbuf = strbuf + '*** ' + string.rjust(str(size),7) + ' ***   '
            if (size > AvailableSPURAM):
                strbuf = strbuf + string.rjust(str(size-AvailableSPURAM),7) + ' bytes OVER BUDGET.'
            else:
                strbuf = strbuf + string.rjust(str(AvailableSPURAM-size),7) + ' bytes left.'
            Log(strbuf)
            for asb in asblist:
                Log( string.rjust(asb,9) + string.rjust( str( GetAsbFileSize(asb) ), 10) )
                
    Log( "" )

    # Report ATX files that are missing
    
    Log( "" )


    gFilesNotFound.sort()
    for filename in gFilesNotFound:
        Log( 'FILE NOT FOUND: ' + filename )
    #for level in levels:
    #    f_atx_filename = kSoundPS2Path + level + '.ATX'
    #    if not os.path.exists(f_atx_filename) :
    #        Log( 'FILE NOT FOUND: ' + level + '.atx' )



def GenerateATXMain():
    gFilesNotFound = []
    Log_Reset()    
    
    # reroute stdout
    #f_out = open(kSoundPS2TempPath + "GenerateATX.txt",'w')
    #console = sys.stdout
    #sys.stdout = f_out

    asbhfiles = glob.glob('.\\*.h')
    for asbhfile in asbhfiles:
        base = os.path.splitext(os.path.basename(asbhfile))[0]
        GenerateATX(base)
    
    Verify_ATXs()

    Log( 'Done.' )

    # restore stdout
    #sys.stdout = console
    #f_out.close()
    
    # result = raw_input("Hit ENTER to continue.")

GenerateATXMain()
