#
#   GenerateSoundList
#
#   input:   
#   output:  "GenerateSoundList.tab"
#
 
import os
import sys
import string
import _ROLparse
import _EPParse
import re
from _ShiftersPaths import *

kOutputFilename = "GenerateSoundList.tab"

def Load_EPParse_tab(table):

	f = open( "_EPParse.tab", 'r' )

	while 1:
		line = f.readline()
		if not line:
			break

		fields = string.split(string.strip(line), "\t")
		cap    = string.lower(fields[0])
		sound  = string.lower(fields[1])
		if (( cmp( cap, 'obstacles')==0 ) or (re.match('spell*', cap)) or (re.match('global*', cap)) ):
			cap='globals'
		
		if ( cmp( cap, 'spell')==0 ):
			cap='globals'
		
		if not ( (cmp( cap, 'test')==0 ) or (cmp( cap, 'verify')==0 ) or (re.match('demo*', cap)) ):
			
			record = (cap,sound)
			if not record in table:
				table.append(record)

	f.close()

def Load_ROLparse_tab(table):

	f = open( "_ROLparse.tab", 'r' )

	while 1:
		line = f.readline()
		if not line:
			break

		fields = string.split(string.strip(line), "\t")
		if len(fields) == 4 :
			cap  = string.lower(fields[0])
			type = string.lower(fields[1])
			if ( type == 'flag' ):
				sound = string.lower(fields[3])
				record = (cap,sound)

				if not record in table:
					table.append(record)
		else:
			print "err: " + line

	f.close

def Load_HardCodedSounds_tab( table ):
	f = open( "_hardcodedSounds.tab", 'r' )
	while 1:
		line = f.readline()
		if not line:
			break

		sound = string.lower(string.strip(line))
		record = ('globals',sound)
		if record not in table:
			table.append( record )
				
	f.close()	

def Load_Textures_nfo( table ):
	f = open( kShiftersRuntimePath + "textures\\textures.nfo" )

	while 1:
		line = f.readline()
		if not line:
			break

		line = string.lower(string.strip(line))
		if (re.match('ricochet',line) or re.match('footstepl',line) or re.match('footstepr',line) or re.match('breakapart',line) ):
			tokens = string.split(line)
			if len(tokens) != 2:
				print "err "+ line
				break
			sound = tokens[1]
			record = ('globals',sound)
			if record not in table:
				table.append( record )
	f.close()
	
def GenerateSoundList():
	
	#
	# delete old output
	#
	f = open(kOutputFilename, 'w')
	f.close()

	#
	# parse EP and ROL files
	#
	_EPParse.ParseAllEPFiles()
	_ROLparse.ROLparseALL()
	
	#
	# parse output from previous step
	#
	data = []
	Load_EPParse_tab(data)
	Load_ROLparse_tab(data)
	Load_HardCodedSounds_tab(data)
	Load_Textures_nfo(data)
	
	data.sort()

	#
	#	write data to "GenerateSoundList.tab"
	#

	f_out = open( kOutputFilename,'w')
	console = sys.stdout
	sys.stdout = f_out

	for record in data:
		cap = record[0]
		sound = record[1]
		print cap + '\t' + sound

	sys.stdout = console
	f_out.close()


if __name__ == '__main__':
	print "Generating \'SoundList.tab\'..."
	GenerateSoundList()
	print "Done."
