#
#   GenerateAsbList
#
#   Generates "asblist.tab", which is a list of which ASB
#   files should be loaded with each cap.
#

import os
import sys
import string
from _ShiftersPaths import *
import _ROLparse
import re

kAsbListTab     = kShiftersRuntimePath + 'script\\asblist.tab'
kLogFile        = '_GenerateAsbList.txt'

gFormAsbs       = (
	"auto",
	"beast",
	"genie",
	"kreegan",
	"hybrid",
	"undead",
    )


def Log_Reset():
    f = open( kLogFile, 'w' )
    f.close()

def Log(str):
    f = open( kLogFile, 'a' )
    stdout_saved = sys.stdout
    sys.stdout = f

    print str

    sys.stdout = stdout_saved
    f.close()

def Load_CapList(caplist):
    f = open(kCapList)
    for line in f.readlines():
        cap = string.split(string.lower(string.strip(line)),'\t')[0]
        if not cap in caplist:
            caplist.append(cap)
    f.close()

def Load_EntityList( entitylist ):
    f = open( "_ROLparse.tab", 'r' )
    for line in f.readlines():
        tokens = string.split(string.lower(string.strip(line)),'\t')
        if len( tokens ) >= 4:
            if ( tokens[1] == "entity" ):
                cap = tokens[0]
                entitytype = tokens[3]
                record = (cap,entitytype)
                if not record in entitylist:
                    entitylist.append(record)
    entitylist.sort()
    f.close()

def Load_EntityAsbTable( entityAsbTable ):
    f = open( "_EntityAsb.tab", 'r' )
    for line in f.readlines():
        tokens = string.split(string.lower(string.strip(line)),'\t')
        if len(tokens) == 2:
            entity = tokens[0]
            asb = tokens[1]
            record = ( entity, asb )
            if not record in entityAsbTable:
                entityAsbTable.append(record)
        else:
            print "error: " + line
    f.close()
    
def FindEntitiesInCap( cap, entitylist, result ):
    rec1 = entitylist[0]
    for record in entitylist:
        entitycap = record[0]
        if cmp(entitycap,cap)==0:
            result.append( record[1] )

def FindEntityAsbName( entityname, entityasbtable ):
    for record in entityasbtable:
        if cmp( record[0], entityname )==0:
            asbname = record[1]
            return asbname
    return ""

def FindFormAsbsUsedByCap( entityAsbsUsedByCap, formAsbsUsedByCap ):
    for formAsb in gFormAsbs:
        for entityAsb in entityAsbsUsedByCap:
            pat = formAsb + '*'
            if ( re.match( pat, entityAsb ) ):
                if formAsb not in formAsbsUsedByCap:
                    formAsbsUsedByCap.append( formAsb )

def DoGenerationOfCapAsbsFile(caplist, entitylist, entityasbtable):

    f_out = open( kAsbListTab,'w')
    console = sys.stdout
    sys.stdout = f_out

    notfound = []
    for cap in caplist:
        entitiesInCap = []
        FindEntitiesInCap( cap, entitylist, entitiesInCap )

        entityAsbsUsedByCap = []
        for entity in entitiesInCap:
            asbname = FindEntityAsbName( entity, entityasbtable )
            if cmp( asbname, "" ) == 0:
                if entity not in notfound:
                    notfound.append( entity )
            else:
                if asbname not in entityAsbsUsedByCap:
                    entityAsbsUsedByCap.append( asbname )

        formAsbsUsedByCap = []
        FindFormAsbsUsedByCap( entityAsbsUsedByCap, formAsbsUsedByCap )
        
        outputline = cap + '\t' + cap + '\t' + string.join( formAsbsUsedByCap, '\t' ) + '\t' + string.join( entityAsbsUsedByCap, '\t' )
        print outputline

    sys.stdout = console
    f_out.close()

    if (len(notfound) > 0):
        notfound.sort()
        Log( "No ASB files found for:" )
        Log( "" )
        for entity in notfound:
            Log("   " + entity)
    
def GenerateAsbList():
    Log_Reset()
    
    caplist = []
    Load_CapList( caplist )

    entitylist = []
    Load_EntityList( entitylist )

    entityAsbTable = []
    Load_EntityAsbTable( entityAsbTable )

    DoGenerationOfCapAsbsFile(caplist, entitylist, entityAsbTable)
    
if __name__ == '__main__':
    GenerateAsbList()

    